package GitLab::Groups;

use utf8;
use strict;
use warnings;
use vars qw($VERSION);

use GitLab::API;
use Log::Any        qw($log);

our $VERSION = v8.12.1;

my $requests = {
    groups  => {
        method      => "GET",
        path        => "/groups",
        query       => [ qw(search) ],
        paginated   => 1,
    },

    group_get_projects => {
        method      => "GET",
        path        => "/groups/<gid>/projects",
        optional    => [ qw(archived visibility order_by sort search ci_enabled_first) ],
        paginated   => 1,
    },

    group_details => {
        method      => "GET",
        path        => "/groups/<gid>",
    },

    group_create => {
        method      => "POST",
        path        => "/groups",
        required    => [ qw(name path) ],
        optional    => [ qw(description visibility_level lfs_enabled request_access_enabled) ],
    },

    transfer_project_to_group => {
        method      => "POST",
        path        => "/groups/<gid>/projects/<projid>",
    },

    group_update => {
        method      => "PUT",
        path        => "/groups/<gid>",
        optional    => [ qw(name path description visibility_level lfs_enabled request_access_enabled) ],
    },

    group_delete    => {
        method      => "DELETE",
        path        => "/groups/<gid>",
    },

    group_members   => {
        method      => "GET",
        path        => "/groups/<gid>/members",
        paginated   => 1,
    },

    group_add_member => {
        method      => "POST",
        path        => "/groups/<gid>/members",
        required    => [ qw!user_id access_level! ],
    },

    group_update_member => {
        method      => "PUT",
        path        => "/groups/<gid>/members/<user_id>",
        required    => [ qw!access_level! ],
    },

    group_delete_member => {
        method      => "DELETE",
        path        => "/groups/<gid>/members/<user_id>",
    },
};

sub import {
    $log->debug("initializing " . __PACKAGE__);
    while (my ($name, $tmpl) = each(%$requests)) {
        $tmpl->{name} = $name unless exists $tmpl->{name};
        GitLab::API->register($tmpl);
    }
}

1;

__END__

=head1 NAME

GitLab::Groups - implements group API calls

See L<GitLab API -- Groups|http://doc.gitlab.com/ce/api/groups.html> for details and
response formats.

=head1 VERSION

Implements API calls for GitLab CE C<v8.10.0>.
Checked 2016-09-29 for GitLab CE C<v8.12.1>.

=head1 DESCRIPTION

=head2 Notation

Please see the documentation for the L<GitLab::Users> module.

Note that not all optional arguments are listed.
Please refer to the official documentation for the full list.

=head2 Group CRUD operations

=over

=item groups()

    $groups = $gitlab->groups( [:search] );

Returns a list of groups.
An optional parameter C<search> can be used to filter the list by substring match on C<name>.

=item group_details()

    $details = $gitlab->group_details( :gid );

Returns details about a group with the given C<gid>.

=item group_create()

    $group = $gitlab->group_create( :name, :path, [:description], [:visibility_level] );

Creates a new group. Required arguments are

    name                the name of the group,
    path                the path for the group (must be unique)

In addition, the following I<optional> arguments can be specified:

    description         group's description
    visibility_level    see GitLab::API for possible values

=item group_update()

    $group = $gitlab->group_update( :gid );

Updates the grup with the given C<gid>.
Takes same arguments as C<group_create>, except they are all optional.

=item group_delete()

    $gitlab->group_delete( :gid );

Deletes the group with the given C<gid>.

=back

=head2 Group members

=over

=item group_members()

    $members = $gitlab->group_members( :gid );

Returns a list of members in the grup with the specified C<gid>.

=item group_add_member()

    $gitlab->group_add_member( :gid, :user_id, :access_level );

Adds a user with C<user_id> to the group with the given C<gid> with the specified C<access_level>.
See L<GitLab::API> module for possible access level values.

=item group_update_member()

    $gitlab->group_update_member( :gid, :user_id, :access_level );

For user with the given C<user_id> the function changes his C<access_level> in the group with the specified C<gid>.

=item group_delete_member()

    $gitlab->group_delete_member( :gid, :user_id );

Removes the user with C<user_id> from the group with the specified C<gid>.

=back

=head2 Group projects and transfer

=over

=item group_get_projects()

    $projects = $gitlab->group_get_projects( :gid );

Returns a list of projects owned by the group with the given C<gid>.

=item transfer_project_to_group()

    $gitlab->transfer_project_to_group( :gid, :projid );

Moves the project with C<projid> to the namespace of the group with the given C<gid>.

I<This method is available only for administrators.>

=back

=head1 AUTHOR

Roman Lacko <L<xlacko1@fi.muni.cz>>

=head1 SEE ALSO

=over

=item L<GitLab>

Wrapper around L<GitLab::API> and other C<GitLab::*> modules.

=back
