package GitLab::Namespaces;

use utf8;
use strict;
use warnings;
use vars qw($VERSION);

use GitLab::API;
use Log::Any        qw($log);

our $VERSION = v8.12.1;

my $requests = {
    namespaces => {
        method      => "GET",
        path        => "/namespaces",
        query       => [ qw(search) ],
        paginated   => 1,
    }
};

sub import {
    $log->debug("initializing " . __PACKAGE__);
    while (my ($name, $tmpl) = each(%$requests)) {
        $tmpl->{name} = $name unless exists $tmpl->{name};
        GitLab::API->register($tmpl);
    }
}

1;

__END__

=head1 NAME

GitLab::Namespaces - extension for Namespaces API

See L<GitLab API -- Namespaces|http://doc.gitlab.com/ce/api/namespaces.html> for details and response formats.

=head1 SYNOPSIS

    use GitLab::API;
    use GitLab::Namespaces;

    my $api = GitLab::API->new();

    # get namespaces exposed to user john.doe
    $api->sudo("john.doe");

    my $namespaces = $api->namespaces(search => "group1");

=head1 VERSION

Implements API calls for GitLab CE C<v8.10.0>.
Checked 2016-09-29 for GitLab CE C<v8.12.1>.

=head1 DESCRIPTION

From GitLab API: I<"Usernames and groupnames fall under a special category called namespaces.">

=over

=item namespaces()

    $namespaces = $gitlab->namespaces();
    $namespaces = $gitlab->namespaces(search => $what);

Returns all namespaces for currently authenticated user.
If an optional argument C<search> is used, only those namespaces that contain the requested string are returned.

To get namespaces of user with login C<$login>, you can do this:

    $gitlab->sudo($login);
    $namespaces = $gitlab->namespaces();

Note that the authenticated user must be an administrator to use L<GitLab::API/sudo>.
If the user with login C<$login> is an administrator, the call will return B<all> namespaces.

=back

=head1 AUTHOR

Roman Lacko <L<xlacko1@fi.muni.cz>>

=head1 SEE ALSO

=over

=item L<GitLab>

Wrapper around L<GitLab::API> and other C<GitLab::*> modules.

=back
