package GitLab::Users;

use utf8;
use strict;
use warnings;
use vars qw($VERSION);

use GitLab::API;
use Log::Any        qw($log);

our $VERSION = 8.12.1;

my $requests = {
    users => {
        method      => "GET",
        path        => "/users",
        query       => [ qw(username search) ],
        paginated   => 1,
    },

    user_by_id => {
        method      => "GET",
        path        => "/users/<uid>",
    },

    user_create     => {
        method      => "POST",
        path        => "/users",
        required    => [ qw(email password username name) ],
        optional    => [ qw(skype linkedin twitter website_url projects_limit
        extern_uid provider bio location admin confirm can_create_group external) ],
    },

    user_update     => {
        method      => "PUT",
        path        => "/users/<uid>",
        # update can cause Conflict, but API always returns 404
        ret         => { 404 => "Not found or Conflict" },
        optional    => [ qw(email password name username skype linkedin twitter
        website_url projects_limit extern_uid provider bio location admin can_create_group external) ],
    },

    user_delete     => {
        method      => "DELETE",
        path        => "/users/<uid>",
    },

    user => {
        method      => "GET",
        path        => "/user",
    },

    self_ssh_keys => {
        method      => "GET",
        path        => "/user/keys",
        paginated   => 1,
    },

    self_get_ssh_key => {
        method      => "GET",
        path        => "/user/keys/<keyid>",
    },

    user_get_ssh_keys => {
        method      => "GET",
        path        => "/users/<uid>/keys",
        paginated   => 1,
    },

    self_add_ssh_key => {
        method      => "POST",
        path        => "/user/keys",
        required    => [ qw!title key! ],
    },

    user_add_ssh_key => {
        method      => "POST",
        path        => "/users/<uid>/keys",
        required    => [ qw!title key! ],
    },

    self_delete_ssh_key => {
        method      => "DELETE",
        path        => "/user/keys/<keyid>",
    },

    user_delete_ssh_key => {
        method      => "DELETE",
        path        => "/users/<uid>/keys/<keyid>",
    },

    self_get_emails => {
        method      => "GET",
        path        => "/user/emails",
        paginated   => 1,
    },

    user_get_emails => {
        method      => "GET",
        path        => "/users/<uid>/emails",
        paginated   => 1,
    },

    self_get_email => {
        method      => "GET",
        path        => "/user/emails/<eid>",
    },

    # not officially supported, may not work :(
    # user_get_email => {
    #     method      => "GET",
    #     path        => "/users/<uid>/emails/<eid>",
    # },

    self_add_email => {
        method      => "POST",
        path        => "/user/emails",
        required    => [ qw!email! ],
    },

    user_add_email => {
        method      => "POST",
        path        => "/users/<uid>/emails",
        required    => [ qw!email! ],
    },

    self_delete_email => {
        method      => "DELETE",
        path        => "/user/emails/<eid>",
    },

    user_delete_email => {
        method      => "DELETE",
        path        => "/users/<uid>/emails/<eid>",
    },

    user_block      => {
        method      => "PUT",
        path        => "/users/<uid>/block",
    },

    user_unblock    => {
        method      => "PUT",
        path        => "/users/<uid>/unblock",
    },
};

sub import {
    $log->debug("initializing " . __PACKAGE__);
    while (my ($name, $tmpl) = each(%$requests)) {
        $tmpl->{name} = $name unless exists $tmpl->{name};
        GitLab::API->register($tmpl);
    }
}

1;

__END__

=head1 NAME

GitLab::Users - implements user API calls

See L<GitLab API -- Users|http://doc.gitlab.com/ce/api/users.html> for details and
response formats.

=head1 VERSION

Implements API calls for GitLab CE C<v8.10.0>.
Checked 2016-09-29 for GitLab CE C<v8.12.1>.

=head1 SYNOPSIS

    use GitLab::API;
    use GitLab::Users;
    # all calls from GitLab::Users are "injected" into the GitLab::API as methods

    my $api = GitLab::API->new();

    my $john = $api->users(username => "john_doe");

    $api->user_create(
        email       => "jane34@somewhere.com",
        password    => "s3cre1",
        username    => "jane.doe",
        name        => "Jane Doe"
    );

=head1 DESCRIPTION

=head2 Notation

All methods take a hash of arguments as specified in the documentation for
L<GitLab::API/exec_request>, but to simplify the documentation,
we will use the following notation:

    users( :username, :search )

which means that the C<users> method takes a hash with two keys, C<username>
and C<search>, so it can be called as

    $gitlab->users(search => "john_doe");

Optional arguments are denoted as C< [:username] >.
Note that not all optional arguments are listed.
Please refer to the official documentation for the full list.

=head2 User CRUD operations

=over

=item users()

    $users = $gitlab->users( [:username], [:search] );

This method returns all users. The optional arguments can be used
to filter out the results:

    username        search for a user by his username (exact match)
    search          search for users with username or email (substring match)

Note that the returned value is still an array even when C<< users(username => $username) >>
always returns at most one result.

=item user_by_id()

    $user = $gitlab->user_by_id( :uid );

Returns a user with the given C<uid>.

=item user_create()

    $user = $gitlab->user_create( :email, :password, :username, :name );

Creates a new user and if succeeds, returns the generated account information.

    email               primary email, must not exist in GitLab,
    password            user password,
    username            login, must not exist in GitLab,
    name                display name

The function can take many optional arguments. Please see the API documentation for
the full list. Most interesting arguments are:

    admin               create an admin account (bool)
    can_create_group    the user will be able to create group (bool)
    external            create an external account (bool)
    projects_limit      maximum number of projects the user can own

=item user_update()

    $user = $gitlab->user_update( :uid );

Updates the user with the given C<uid>. It takes the same arguments as C<user_create>
except they are all optional. Note that though it is possible to change user's primary e-mail
and username, it is probably not a very sane thing to do.

Also, from documentation:

    Note, at the moment this method does only return a 404 error, even in cases where
    a 409 (Conflict) would be more appropriate, e.g. when renaming the email address to some existing one.

=item user_delete()

    $gitlab->user_delete( :uid );

Deletes the user with the given C<uid>.

Note that

    calling this function for a non-existent user id still returns a status code 200 OK.
    The JSON response differs if the user was actually deleted or not.

(from the GitLab API documentation).

=item user()

    $user = $gitlab->user();

Returns the user whose authentication key is used for the request
(or impersonated user if C<sudo> is in effect).

Behaves the same as the L<GitLab::API/whoami> method.

=back

=head2 SSH keys

=over

=item self_ssh_keys()

    $keys = $gitlab->self_ssh_keys();

Returns a list of SSH keys for the user that makes the request.

=item self_get_ssh_key()

    $key = $gitlab->self_get_ssh_key( :keyid );

For the user that makes the request, the function returns the SSH key with the given C<keyid>.

=item user_get_ssh_keys()

    $keys = $gitlab->user_get_ssh_keys( :uid );

Returns a list of SSH keys for the user with the given C<uid>.

=item self_add_ssh_key()

    $key = $gitlab->self_add_ssh_key( :title, :key );

Adds a new key for the user that makes the request. Required arguments are

    title               the title of the key
    key                 public SSH key

=item user_add_ssh_key()

    $key = $gitlab->user_add_ssh_key( :uid, :title, :key );

Similar to C<self_add_ssh_key>, but targets the user with the given C<uid>.

=item self_delete_ssh_key()

    $gitlab->self_delete_ssh_key( :keyid )

Removes the SSH key with the given C<keyid> for the user that makes the request.

=item user_delete_ssh_key()

    $gitlab->user_delete_ssh_key( :uid, :keyid );

Removes the SSH key with the C<keyid> for the user with the given C<uid>.

=back

=head2 E-mails

=over

=item self_get_emails()

    $emails = $gitlab->self_get_emails();

Returns a list of B<secondary> e-mails for the user that makes the request.

Note that primary e-mail can be obtained from the result of C<whoami>.

=item user_get_emails()

    $emails = $gitlab->user_get_emails( :uid );

Returns a list of B<secondary> e-mails for the user with the C<uid>.

Note that primary e-mail can be obtained from the result of C<user_by_id>.

=item self_get_email()

    $email = $gitlab->self_get_email( :eid );

Returns the e-mail with the given C<eid> of the user that makes the request.

=item self_add_email()

    $email = $gitlab->self_add_email( :email );

Adds a new C<email> for the user that makes the request. The e-mail must
not exist in GitLab.

=item user_add_email()

    $email = $gitlab->user_add_email( :uid, :email );

Adds a new C<email> for the user with the given C<uid>. The e-mail must
not exist in GitLab.

=item self_delete_email()

    $gitlab->self_delete_email( :eid );

Deletes the e-mail with C<eid> of the user that makes the request.

=item user_delete_email()

    $gitlab->user_delete_email( :uid, :eid );

Removes the e-mail with C<eid> of the user with the given C<uid>.

=back

=head2 Account blocking

=over

=item user_block()

    $gitlab->user_block( :uid );

Blocks the user with C<uid> in the GitLab.

=item user_unblock()

    $gitlab->user_unblock( :uid );

Unblocks the user with C<uid> in the GitLab.
Note that LDAP blocked users cannot be unblocked by this function.
In that case the API would return C<403 Forbidden>.

=back

=head1 AUTHOR

Roman Lacko <L<xlacko1@fi.muni.cz>>

=head1 SEE ALSO

=over

=item L<GitLab>

Wrapper around L<GitLab::API> and other C<GitLab::*> modules.

=back
